;------------------------------------------------------------------------------
; File Name :      adc41262.inc
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 22, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; ADC model refinement for MCU described in DS 41262
; 
;------------------------------------------------------------------------------

 ; enum ConvClock
 constant ADC_eClockFOSC_2      = b'000'
 constant ADC_eClockFOSC_8      = b'001'
 constant ADC_eClockFOSC_32     = b'010'
 constant ADC_eClockFRC         = b'011'
 constant ADC_eClockFOSC_4      = b'100'
 constant ADC_eClockFOSC_16     = b'101'
 constant ADC_eClockFOSC_64     = b'110'

ADC_mSetClock macro ConvClock
    if ConvClock > ADC_eClockFOSC_64  + 1
        error Improper parameter passed to "ADC_mSetClock" 
    endif
    _sfr_banksel    ADCON1
    movlw           (ConvClock << ADCS0)
    movwf           ADCON1
 endm

ADC_mSetVref macro VrefSrc
 if VrefSrc != ADC_eVrefVdd && VrefSrc != ADC_eVrefVref
    error Improper parameter passed to "ADC_mSetVref" 
 endif
 if VrefSrc == ADC_eVrefVdd
    bcf         ADCON0, VCFG
 endif
 if VrefSrc == ADC_eVrefVref
    bsf         ADCON0, VCFG
 endif
 endm

ADC_mSetChannel macro const
    _sfr_banksel    ADCON0
    movlw   ~((1 << CHS3) | (1 << CHS2) | (1 << CHS1) | (1 << CHS0))
    andwf   ADCON0,W
    iorlw   (const & 0Fh) << CHS0
    movwf   ADCON0
 endm

ADC_mSetChnlDyn macro gpr
    rlf         gpr, F
    rlf         gpr, W
    clrc
    rrf         gpr, F
    andlw       0Fh
    _sfr_banksel    ADCON0
    bcf         ADCON0, CHS3
    bcf         ADCON0, CHS2
    bcf         ADCON0, CHS1
    bcf         ADCON0, CHS0
    iorwf       ADCON0, W
    movwf       ADCON0
 endm

ADC_mON macro
    _sfr_banksel    ADCON0    
    bsf         ADCON0, ADON
 endm

ADC_mOFF macro
    _sfr_banksel    ADCON0    
    bcf         ADCON0, ADON
 endm

ADC_mSetResJustf macro Justf
 if Justf != ADC_eJustfRight && Justf == ADC_eJustfLeft
    error Improper parameter passed to "ADC_mSetResJustf"
 endif
    _sfr_banksel ADCON0
 if Justf == ADC_eJustfRight
    bsf         ADCON0, ADFM
 endif
 if Justf == ADC_eJustfLeft
    bcf         ADCON0, ADFM
 endif
 endm

ADC_mGO macro
    _sfr_banksel    ADCON0    
    bsf         ADCON0, GO
 endm

ADC_mSkipDone macro
    _sfr_banksel    ADCON0    
    btfsc       ADCON0, GO
 endm

ADC_mSkipNotDone macro
    _sfr_banksel    ADCON0    
    btfss       ADCON0, GO
 endm

ADC_mSetAnaloginput macro const
  _sfr_banksel  TRISA
  if const <= AN3
   if const < AN3
    bsf         TRISA, const
   else
    bsf         TRISA, RA4
   endif
  if const >= AN4 && const <= AN9
   if const <= AN7
    bsf         TRISC, const - AN4
   else
    bsf         TRISC, RC6 + const - AN8
   endif
  endif
  if const >= AN10
    bsf         TRISB, RB4 + const - AN10    
  endif
  if const <= AN7
   _sfr_banksel    ANSEL
    movlw       .1 << const
    iorwf       ANSEL
  endif
  if const >= AN8
    _sfr_banksel    ANSELH
    movlw       .1 << (const - AN8)
    iorwf       ANSELH
  endif
 endm
