;------------------------------------------------------------------------------
; File Name :      hla.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 21, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; Hall Light Automaton implementation
; 
;------------------------------------------------------------------------------
#define __hla_asm
 include "pic.inc"
 include "vos.inc"
 include "hla.inc"
 include "act.inc"
 include "msw.inc"

;------------------------------------------------------------------------------
VOS_Classes             ; Actuator class dispatch table
HLASW VOS_mClass HLA_Handler, HLA_Init      ; Switch automat
HLAPB VOS_mClass HLA_Handler, HLA_Init      ; Push-button automat
HLATB VOS_mClass HLA_Handler, HLA_Init      ; Toggle-button automat
;------------------------------------------------------------------------------
; Design decisions
; 1. Two kinds of inputs are supported:
;   1a - plain two-position switch
;   1b - push-button (spring-return switch)
; 1.1. In 1a mode every change in the switch states causes sending 'toggle'
; 1.2. In 1b mode any changes to non-zero mode causes sending 'on'
; 1.3. Modes are distingushed by the class reference
; 2. Any output message is sent to the 'client' and to a statically bind
;    message delay line (defined as extern)

 constant hla_cMdlOff    = .0
    udata_ovr
hla_mdl         res 1       ; temporary storage for the handle of message delay line

HLA_code code
HLA_Handler
    movlw       MSW_cMsgIDChange
    xorwf       INDF, W
    skpz
    return
    incf        FSR, F
    movfw       This_class
    xorlw       HLASW                   ; if This.getClass == HLASW
    bz          _sw                     ;   process both on and off postions of switches
    tstf        INDF                    ; Other classes process only ON state
    skpnz                               ; and ignore off state
    return                              ; 
_sw PDD_mSendToAcceptor ACT_cMsgIDActuate
    movfw       This_class
    xorlw       HLAPB                   ; if This.getClass == HLAPB
    movlw       ACT_cValueON            ;   send ON mesage
    skpz                                ; else
    movlw       ACT_cValueALT           ;   send ALT message
    movwf       INDF
    PDD_mReadObjData hla_cMdlOff
    banksel     hla_mdl
    movwf       hla_mdl
    VOS_mPostMsgIndir hla_mdl, ACT_cMsgIDActuate
    clrf        INDF
    return
HLA_Init    
    return
  end  
