;------------------------------------------------------------------------------
; File Name :     main248.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 17, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
; MIHA - Multi Input Hall Automaton
;------------------------------------------------------------------------------

 include "pic.inc"
 include "vos.inc"
 include "msw.inc"
 include "hla.inc"
 include "act.inc"
 include "mdl.inc"
 include "adc.inc"
 include "ppi.inc"
 if VOS_cDEBUG == true
 include "dbg.inc"
 endif
;------------------------------------------------------------------------------
	CONFIG OSCS = OFF, OSC = HS, BOR = OFF, PWRT = ON, WDT = ON, WDTPS = 1
	CONFIG STVR = ON, LVP = OFF, DEBUG = OFF
;------------------------------------------------------------------------------
  ; RA0 - potentiometer, RA1-RA6 - inputs
  ; RC0 - RC3 - outputs, RC4-RC7 - inputs
  constant mswPORT = PORTA
  constant mswMASK = b'1111110'
  constant mswBufferSize = .6 + .1
  constant mswTRIS = TRISA
  constant actPORT = LATC
  constant actMASK = b'00000001'
  constant actTRIS = TRISC
  constant mswPORT1= PORTC
  if VOS_cDEBUG == true
  constant mswMASK1= b'11110000' & ~((1<<DBG_ALR) | (1<<DBG_SIG) | (1 << DBG_TX))
  else
  constant mswMASK1= b'11110000'
  endif
  constant mswBufferSize1 = .4 + .1
  constant ppiChannel = AN0

  constant mswPORT3 = PORTA
  constant mswMASK3 = b'00001000' ; RA3


;------------------------------------------------------------------------------
; Object instantiation
VOS_Objects                                             ; starts VOS object section

switchDebouncer
    VOS_mPersistentObject MSW, switchDebouncerData      ; creates an instance of MSW class 

hallAutomatonForSwitches
    VOS_mPersistentObject HLASW, hallAutomatonSWData

messageDelayLine     
    VOS_mPersistentObject MDL, messageDelayData

actuator
    VOS_mPersistentObject ACT, actuatorData

parameterInput
    VOS_mPersistentObject PPI, parameterInputData

; Additional objects only on more capable chips
switchDebouncerSecondary
    VOS_mPersistentObject MSW, switchDebouncerSecondaryData

switchDebouncer3
    VOS_mPersistentObject MSW, switchDebouncer3Data


hallAutomatonForPushButtons
    VOS_mPersistentObject HLAPB, hallAutomatonPBData

hallAutomaton3
    VOS_mPersistentObject HLATB, hallAutomaton3Data


;------------------------------------------------------------------------------
; Object data and commutation
DEEPROM code_pack
switchDebouncerData
    MSW_mData hallAutomatonForSwitches, mswPORT, mswMASK, switchDebouncerBuffer

 ifdef switchDebouncerSecondary
switchDebouncerSecondaryData
    MSW_mData hallAutomatonForPushButtons, mswPORT1, mswMASK1, switchDebouncerBuffer1
 endif
 ifdef switchDebouncer3
switchDebouncer3Data
    MSW_mData hallAutomaton3, mswPORT3, mswMASK3, switchDebouncerBuffer3
 endif

hallAutomatonSWData                             ; Automaton connected to:
    HLA_mData actuator, messageDelayLine        ; and to the message delay line

 ifdef hallAutomatonForPushButtons
hallAutomatonPBData
    HLA_mData actuator, messageDelayLine
 endif

 ifdef hallAutomaton3
hallAutomaton3Data
    HLA_mData actuator, messageDelayLine
 endif

messageDelayData
    MDL_mData actuator

actuatorData
    ACT_mData actPORT, actMASK

parameterInputData
    PPI_mData messageDelayLine, MDL_cMsgIDSetParam, ppiChannel

;------------------------------------------------------------------------------
; RAM allocation
    udata
    global MSW_Buffers
MSW_Buffers
switchDebouncerBuffer res mswBufferSize    
 ifdef switchDebouncerSecondary
switchDebouncerBuffer1 res mswBufferSize1
 endif
 ifdef switchDebouncer3
switchDebouncerBuffer3 res .2
 endif

;------------------------------------------------------------------------------
 constant miha_PCFG    = b'1110'    ; ANS0 == Analog, the others are digital
 constant miha_TRISAN  = .1         ; ANS0 == RA0
;------------------------------------------------------------------------------
 if VOS_cDEBUG == true
strings code
VOS_banner dt  "VELOOS v.1.1  eugene@hutorny.in.ua",0DH,0AH,0
 endif
; Application initialization
VOS_BootHandlers code                           ; starts a section for the initialization routine
miha_init
    movlw       miha_PCFG
    iorwf       ADCON1,F
    movlw       b'111'
    movwf       CMCON                           ; comparator OFF

  if VOS_cDEBUG == true
    movlw       (1 << DBG_TX)
    movwf       DBGIO
    movlw       ~((1<<DBG_ALR) | (1<<DBG_SIG) | (1 << DBG_TX))
    banksel     DBG_TRIS
    andwf       DBG_TRIS, F
    DBG_mPrint  VOS_banner
  endif
 end
