;------------------------------------------------------------------------------
; File Name :      msw.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 15, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; Multi-input switch
; 
;------------------------------------------------------------------------------
; Design decisions
; Debouncing strategy - RC-integrator implemented in software

#define __msw_asm
 include "pic.inc"
 include "vos.inc"
 include "msw.inc"

; constants
 constant msw_cPortAddrOff  = .0
 constant msw_cPortMaskOff  = .1
 constant msw_cIBOff        = .2

VOS_Classes
MSW VOS_mClass MSW_Handler, MSW_Init

    udata_ovr
msw_mask        res 1
msw_maskstate   res 1
msw_data        res 1
msw_iter        res 1
msw_newstate    res 1

 if MSW_cUseMajority
msw_majority res .3     ; (a & b) | (b & c) | (a & c)
 endif

msw_mLoadPort macro
    bankisel    PORTA
    PDD_mReadObjData    msw_cPortAddrOff
 endm

msw_mLoadMask macro
    PDD_mReadObjData msw_cPortMaskOff
 endm

msw_mLoadIB macro
    extern      MSW_Buffers
    bankisel    MSW_Buffers
    PDD_mReadObjData msw_cIBOff
 endm

msw_mReadPort macro
    msw_mLoadPort
    movwf       FSR
    movfw       INDF
 if MSW_cUseMajority
    movwf       msw_majority+0
    movwf       msw_majority+1
    movfw       INDF
    movwf       msw_majority+1
    andwf       msw_majority+2, F
    movfw       INDF
    andwf       msw_majority+2, F
    andwf       msw_majority+0, W
    iorwf       msw_majority+1, W
    iorwf       msw_majority+2, W
 endif   
 endm

MSW_code code

MSW_Handler
    movlw       VTMR_cMsgIDTimer1ms
    xorwf       INDF, W
    skpz
    return
    ; goto msw_Activity

msw_Activity
    _pic_bankinit
msw_ReadPort        ; read port specified in the constructor
    msw_mLoadMask
    banksel     msw_mask    
    movwf       msw_mask
    movwf       msw_maskstate
    _pic_bankinit
    msw_mLoadPort
    movwf       FSR   
    banksel     msw_mask
    msw_mReadPort
    andwf       msw_mask, W
    movwf       msw_data
    ; goto msw_Integrate

msw_Integrate       ; integrate read data
    msw_mLoadIB
    movwf       FSR
    movlw       .1
    movwf       msw_iter
    movff       INDF, msw_newstate
    incf        FSR, F
_0  clrc
    rrf         msw_mask, F
    bnc         _1
    call        msw_IntegrateChannel
    incf        FSR,F
    clrc
_1  rlf         msw_iter, F
    rrf         msw_data, F
    tstf        msw_mask
    bnz         _0
    ; bra      msw_Post
    
msw_Post
    msw_mLoadIB
    movwf       FSR
    movfw       msw_newstate
    xorwf       INDF, W
    skpnz  
    return      ; state is not changed
    movfw       msw_newstate
    movwf       INDF
    PDD_mSendToAcceptor   MSW_cMsgIDChange
    banksel     msw_mask
    movfw       msw_newstate
    xorwf       msw_maskstate, W  ; invert the value (since switch active state == 0)
    movwf       INDF
    return
    
msw_IntegrateChannel    ; void msw_IntegrateChannel(state msw_data[0]; integral * FSR);
    btfss       msw_data,.0
    bra         _d
    btfsc       INDF, .4        ; if INDF >= 16
    bra         _on
    incf        INDF, F
    movlw       msw_cLevelOn    ; threasholds are 12 and 3
    subwf       INDF, W         ; W := INDF - W; c == INDF >= msw_cLevelOn
    skpc
    return
_on movfw       msw_iter
    iorwf       msw_newstate, F
    return
_d  tstf        INDF
    bz          _of
    decf        INDF, F
    movfw       INDF
    sublw       msw_cLevelOff   ; W := msw_cLevelOff - INDF; c == INDF <= msw_cLevelOff
    skpc
    return
_of comf        msw_iter, W
    andwf       msw_newstate, F    
    return

MSW_Init
    _pic_bankinit
    msw_mLoadPort
    movwf       FSR
    pic_mPortToTris
    msw_mLoadMask
    banksel     msw_iter
    movwf       msw_iter
    movwf       msw_mask
    iorwf       INDF, F
    _pic_bankinit
    msw_mLoadPort
    movwf       FSR
    movfw       INDF                    ; read initial switch states
    andwf       msw_mask,F              ; apply mask
    msw_mLoadIB
    movwf       FSR
    banksel     msw_mask
    movff       msw_mask, INDF          ; msw_state == PORT & MASK
    incf        FSR,F
_9  tstf        msw_iter
    skpnz
    return
    clrc
    rlf         msw_iter, F
    skpc
    bra         _9
    clrf        INDF
    incf        FSR,F
    bra         _9
 end
