;------------------------------------------------------------------------------
; File Name :      ppi.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 24, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
; PDO - Process Data Object 
; An abstract class
;------------------------------------------------------------------------------
; PPI - Potentiometer Parameter Input implementation
; 
;------------------------------------------------------------------------------
; Design decision
; 1. PPI requires exclusive access to ADC
; 2. multiple instantes of PPI uses time-quanting, 
; 2.1. E.g. each instance uses certain timer value to start conversion
; 2.2. Interrupt is not used, results are polled after estmated time
; 2.3. All acquisition will be performed in two timer events:
; 2.3.1. On first event ADC is connected to a proper channel
; 2.3.2. On second event concersion is perfromed (2-6 us)
; 2.4. Between setting up ADC channel and starting conversion there will be 
;      at least 1 ms which is more than enough for acquisition
; 4. Channel is specified in constructor and is used setting CHSx and ANSx
; 5. ADC will operate on FRC, Vref = Vss/Vdd
; 6. 10-bit conversion adjusted right is used
; 7. Application is responsible for setting proper TRIS bit as input
; 8. Both object and message ID are specifed in constructor
;------------------------------------------------------------------------------
#define __ppi_asm
 include "pic.inc"
 include "vos.inc"
 include "ppi.inc"
 include "adc.inc"
;------------------------------------------------------------------------------

 constant ppi_cMsgIDOff  = .0
 constant ppi_cChnlOff   = .1
 constant ppi_cQuantumMask = 03Fh        ; Channels will be acquired every (ppi_cQuantumMask + 1) ms

 constant ppi_cMsgIDGO  = 33h

;------------------------------------------------------------------------------
    udata_ovr
ppi_chan        res 1
ppi_msgid       res 1
;------------------------------------------------------------------------------

VOS_Classes             ; Actuator class dispatch table
PPI VOS_mClass PPI_Handler, PPI_Init
;------------------------------------------------------------------------------

PPI_code    code
    
PPI_Handler
    switch      INDF
    case        VTMR_cMsgIDTimer1ms, ppi_time1ms
    case        ppi_cMsgIDGO, ppi_acquire
    return
ppi_time1ms
    PDD_mReadObjData ppi_cChnlOff
    banksel     ppi_chan
    movwf       ppi_chan
    incf        FSR,F
    movlw       ppi_cQuantumMask
    andwf       INDF, W
    xorwf       ppi_chan, W
    skpz
    return
ppi_setChannel
    ADC_mSetChnlDyn ppi_chan                            ; select ADCc channel
    VOS_mPostMsgIndir This_handle, ppi_cMsgIDGO         ; post a message to itself
    return
ppi_acquire
    ADC_mGO             ; approx 700 IC between setting the channel and starting conversion
_1  ADC_mSkipDone
    bra _1
    banksel     ppi_msgid
    PDD_mReadObjData ppi_cMsgIDOff   
    movwf       ppi_msgid
    PDD_mSendToAcceptorDyn  ppi_msgid
    _sfr_banksel ADRESL
    movff       ADRESL, INDF            ; ppi_message.valueL := ADRESL;
    incf        FSR,F
    _sfr_banksel ADRESH
    movff       ADRESH, INDF            ; ppi_message.valueH := ADRESH;
    return
;------------------------------------------------------------------------------
PPI_Init
    return

;------------------------------------------------------------------------------
VOS_BootHandlers code
PPI_OnBoot
    _pic_bankinit
    ADC_mSetClock       ADC_eClockFRC
    ADC_mSetResJustf    ADC_eJustfRight                
    ADC_mSetVref        ADC_eVrefVdd
    ADC_mON
  end
