;------------------------------------------------------------------------------
; File Name :      vos.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.04
;      Date:      Dec 03, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
; VELOOS implementation
;------------------------------------------------------------------------------
; Revision history
; 1.02 - Fixed vos_mIncMsgPtr for queues ending on a bank edge
; 1.03 - Added pagesel after hadlers calls
; 1.04 - Added to ISR saving/restoring of FSR0H and BSR
#define __vos_asm
 include "pic.inc"
 include "vos.inc"

vos_queuesize equ VOS_cMessageQueueSize * VOS_cMessageSize

;------------------------------------------------------------------------------
; Main Message Queue
;------------------------------------------------------------------------------
;
 global vos_messageQueue, vos_messageQueueEnd
vos_queues udata _pic_maxram - vos_queuesize + 1
vos_messageQueue res vos_queuesize
vos_messageQueueEnd
 constant vos_queueEnd = _pic_maxram + 1
;------------------------------------------------------------------------------
; VOS system registers
;------------------------------------------------------------------------------
; Internal registers
 global vos_msgptr, vos_putptr, vos_currObj
vos_regs udata
vos_msgptr      res 1    
vos_putptr      res 1
 if VOS_cEnableIdle
vos_idleObj     res 1       ; next object to send the idle message
 endif
  if VOS_cDEBUG
vos_timeOutObj  res 1
  endif

;------------------------------------------------------------------------------
; Shared bank registers
 global vos_currClass
 if _pic_shrram >= .4
vos_shr udata_shr
 else
vos_shr udata
 endif

This_handle
vos_currObj     res 1 

This_class
vos_currClass   res 1                   ; these should be kept with This

This            res 1

VOS_FLAGS       res 1
vos_tmp         res 1                   ; vos temporary storage

;------------------------------------------------------------------------------
; Temporary storage registers for the interrupt routine
 if _pic_shrram >= .7
TMPISR udata_shr
 else
TMPISR udata
 endif
TMP_WREG        res 1
TMP_STATUS      res 1
TMP_FSR         res 1
 if _pic_fam == _pic16
  if _pic_maxrom >= 0800h
TMP_PCLATH      res 1
  endif
 endif

 if _pic_fam == _pic18
TMP_FSRH        res 1
TMP_FSR0L       res 1
TMP_FSR0H       res 1
TMP_BSR         res 1
TMP_BSR0        res 1
 endif
;------------------------------------------------------------------------------
; Implementation macros
vos_mIncMsgPtr macro ptr                ; W - implicit parameter
    movlw       VOS_cMessageSize
    addwf       ptr, F
 ;+02 Special cases for queues ending on a bank edge
 if low(vos_queueEnd) == .0
    ; C := ptr >= vos_queueEnd 
 else
    movlw       vos_messageQueueEnd     ; W := vos_messageQueueEnd - ptr
    subwf       ptr, W
 endif
    movlw       vos_messageQueue        
    skpnc                               ; C := ptr < vos_messageQueueEnd
    movwf       ptr    
 endm 

vos_mInvokeHandler macro
   if VOS_cInvoke == VOS_eInvokeCall
    rcall       vos_CallHandler
    pagesel     vos_Operational
   else
    bra         vos_CallHandler
   endif
 endm

vos_mClearStack macro
  if _pic_fam == _pic18
    ; Place for adding post-mortal stack dump
    clrf        STKPTR
  endif
 endm

;------------------------------------------------------------------------------

 if (_pic_fam == _pic16) || (_pic_fam == _pic12)
WDT_TO      equ STATUS
 endif   
 if _pic_fam == _pic18
WDT_TO      equ RCON
 endif   


; Create object
; Execute objects
; Post event
; Process event
; Object queue
; Message queue

    org 000h
; -02. DS says that PCLATH cleared WDT reset
; if ((_pic_fam == _pic16) || (_pic_fam == _pic12)) && (_pic_maxrom >= 0800h)
;    clrf        PCLATH          ; PCLATH is not know on WDT reset
; endif
    goto        vos_Reset
;------------------------------------------------------------------------------
; Main System state machine: Reset, Boot, Timeout, Operational, Broadcast, Idle
;------------------------------------------------------------------------------
VOS_StateMachine code
vos_Reset
	btfss	    WDT_TO, NOT_TO	; Timeout detected
	bra         vos_Timeout     ; Handle Timeout
	;bra 	    vos_Boot        ; Enter boot state

;------------------------------------------------------------------------------
vos_Boot
    ; init WDT
    rcall       vos_InitSFR
    rcall       vos_InitQueue
    rcall       vos_InitSystem

vos_mClrwdt macro
    clrwdt
  if VOS_cEnableSoftWDT
    bcf         VOS_bSoftWDT
  endif
 endm

;------------------------------------------------------------------------------
vos_Operational
    banksel     vos_msgptr
    vos_mClrwdt
    ifdef PEIE
    bsf         INTCON, PEIE            ; just in case application has forgotten to return it back
    endif
    ei                                          
    FSR_setfl   vos_msgptr, vos_messageQueue      ; setup FSR pointing to message queue
    ; [!] assuming FSR_setfl leaves W loaded with pointer
    btfsc       VOS_bQueueEmpty         ; if queue is not empty
    bra         vos_Idle
    ;vos_CallHooks
    if VOS_cEnableBroadcast
    movlw       VOS_cBCastID
    xorwf       INDF, W
    skpnz
    bra         vos_Broadcast
    endif
    movfw       INDF
    banksel     vos_currObj    
    movwf       vos_currObj    
    rcall       vos_LoadObject
    skpnz
    bra         vos_nextmessage
    incf        FSR,F
    vos_mClrwdt
    vos_mInvokeHandler
vos_HandlerExit
    ei          ; just in case handler has turend it off
    ; TODO: if Queue is overflown - print to a debug console
    global      vos_HandlerExit
vos_nextmessage
    rcall       vos_IncMsgPtr
    bra         vos_Operational

;------------------------------------------------------------------------------
 if VOS_cEnableBroadcast
vos_Broadcast
    bsf         VOS_bStateBCast
    loadf       vos_currObj, scnstart_low VOS_ObjectSection
bc0 rcall       vos_GetNextObj
    skpnc
    bra         vos_BroadcastDone
    skpnz
    bra         bc1
    bankisel    vos_messageQueue
    banksel     vos_msgptr
    incf        vos_msgptr, W    
    movwf       FSR                 ; setup FSR pointing to the message.msgid
    rcall       vos_CallHandler
vos_BCastExit
    global      vos_BCastExit
    banksel     vos_currObj
bc1 incf        vos_currObj, F
    incf        vos_currObj, F
    bra         bc0
vos_BroadcastDone
    rcall       vos_IncMsgPtr
    bcf         VOS_bStateBCast
    bra         vos_Operational    
 endif

;------------------------------------------------------------------------------
vos_Idle
  if VOS_cEnableIdle
    VOS_mPostMsgIndir vos_idleObj, VOS_cMsgIDIdle
    incf        vos_idleObj, F
    incf        vos_idleObj, F
    movlw       scnstart_low VOS_ObjectSectionEnd
    subwf       vos_idleObj,W
    movlw       scnstart_low VOS_ObjectSection
    skpnc
    movwf       vos_idleObj
  endif
    bra         vos_Operational

;------------------------------------------------------------------------------
    global      vos_Timeout
vos_Timeout 
	bsf	    WDT_TO, NOT_TO	        ; Clear system timout flag
    ; [!] restore the operational state
    ; vos_currObj is pointing to faulty objetc
  if VOS_cDEBUG
    banksel vos_currObj    
    movff   vos_currObj, vos_timeOutObj
    ; TODO: dump vos_timeOutObj to a debug console
  endif
    bsf     VOS_bTimeoutFlag
    vos_mClrwdt
    vos_mClearStack
    vos_mJumpToExit

;------------------------------------------------------------------------------
; VOS internal routines
;------------------------------------------------------------------------------
VOS_InternalRoutines code

;------------------------------------------------------------------------------
vos_IncMsgPtr
    banksel vos_msgptr
    vos_mIncMsgPtr vos_msgptr
    movfw       vos_msgptr
    di
    bcf         VOS_bQueueFull
    xorwf       vos_putptr, W
    skpnz
    bsf         VOS_bQueueEmpty
    ei
    return

; constant vos_cDispEntries = .2
; if VOS_cPersistentLocation  == VOS_ePLEeprom 
;VOS_cMaxObjectSpace = VOS_cMaxObjects * vos_cDispEntries
; endif
;;------------------------------------------------------------------------------
vos_GetNextObj ; returns C - End of list, Z - null class
    movlw       scnstart_low VOS_ObjectSectionEnd
    ;movlw       scnstart_low VOS_ObjectSection
    ;addlw       VOS_cMaxObjectSpace
    subwf       vos_currObj,W           ; W := vos_currObj - VOS_ObjectSectionEnd
    skpnc                               ; B := vos_currObj - VOS_ObjectSectionEnd < 0
    return                              ; C := vos_currObj >= VOS_ObjectSectionEnd
    movfw       vos_currObj
;    bra     vos_LoadObject
    
;------------------------------------------------------------------------------
vos_LoadObject ; Z - null class
    rcall       vos_ReadObject
    banksel     vos_currClass
    movwf       vos_currClass
    rcall       vos_ReadNextObj
    banksel     This
    movwf       This
    clrc
    movlw       0FFh
    xorwf       vos_currClass,W
    skpnz
    return         
    tstf        vos_currClass
    return         


 ifdef EEPGD
   ;variable vos_cReadROM = VOS_cPersistentLocation == VOS_ePLRom
   variable vos_cReadROM = false
 else
   variable vos_cReadROM = false
 endif

vos_mSetEEPGD macro
  ifdef EEPGD
    _pic_banksel EECON1
    if vos_cReadROM
      bsf       EECON1, EEPGD
    else
      bcf       EECON1, EEPGD
    endif
  endif
 endm
     
 if (VOS_cPersistentLocation == VOS_ePLEeprom) || vos_cReadROM
;------------------------------------------------------------------------------
; byte vos_ReadObject(void * near vos_currObj, void * near W)
vos_ReadObject
;------------------------------------------------------------------------------
; byte vos_ReadEEPROM(void * near W);
vos_ReadEEPROM
    _pic_bankinit
    vos_mSetEEPGD
    _pic_banksel EEADR
	movwf	    EEADR		; EEADDR = W
    ifdef EEADRH
    if vos_cReadROM
    movlw       scnstart_high VOS_ObjectSection
    movwf       EEADRH
    endif
    endif
vos_ReadEEPROM1
    _pic_banksel EECON1
	bsf 	    EECON1, RD 	; EE Read	
    _pic_banksel EEDATA
	movfw 	    EEDATA   	; W = EEDATA
    return

;------------------------------------------------------------------------------
vos_ReadNextObj
;------------------------------------------------------------------------------
vos_ReadNextEEPROM
    _sfr_banksel EEADR      
    ifdef       EEADRH
    incfsz      EEADR, F
    decf        EEADRH, F
    incf        EEADRH, F
    else
    incf        EEADR, F
    endif
    bra         vos_ReadEEPROM1    
                                errorlevel +302
 else
;------------------------------------------------------------------------------
vos_ReadObject
;------------------------------------------------------------------------------
; byte vos_ReadROM(void * near W);
vos_ReadROM
    movlw       scnstart_high VOS_ObjectSection
    _sfr_banksel    PCLATH    
    movwf       PCLATH
    banksel     vos_currObj
    movfw       vos_currObj
    movwf       PCL
;------------------------------------------------------------------------------
vos_ReadNextObj
;------------------------------------------------------------------------------
vos_ReadNextROM
    banksel     vos_currObj
    incf        vos_currObj, W
    movwf       PCL
 endif

;------------------------------------------------------------------------------
; VOS Messaging routines
;------------------------------------------------------------------------------
 global vos_PostMsg, vos_LoadPutPtr
vos_LoadPutPtr                          ; FSR = vos_putptr; ; (vos_putptr += msgSize) &= msgqueuesize
    btfsc       VOS_bQueueFull
vos_QueueOverflown
    bsf         VOS_bQueueOverflow
    banksel     vos_putptr
    bankisel    vos_messageQueue
    movff       INTCON, vos_tmp         ; preserve GIE
    di          ; critical section
    movff       vos_putptr, FSR
    vos_mIncMsgPtr vos_putptr
    movfw       vos_putptr
    xorwf       vos_msgptr, W
    skpnz
    bsf         VOS_bQueueFull
    btfsc       vos_tmp, GIE            ; restore GIE        
    ei          ; critical section end  ; Interrupt latency 14 IC
    bcf         VOS_bQueueEmpty
    return

vos_PostMsg     ; byte* vos_PostMsg(W msgid)
    ;TODO: if VOS_bQueueOverflow
    ; raise exception (abnormal routine interrupt)
    incf        FSR, F
    movwf       INDF
    movlw       VOS_cMessageSize - .2   ; move to the last byte, consider that already incremented once
    addwf       FSR, F
    movff       This, INDF
    movlw       -(VOS_cMessageSize - .3)
    addwf       FSR, F                  ; move to the first user location
    return

;------------------------------------------------------------------------------
; VOS Object invocation
;------------------------------------------------------------------------------
   variable _vos_cDispEntitySize = .1
   if _pic_fam >= _pic18
_vos_cDispEntitySize = .2
   if VOS_cDispScope == VOS_eDispFar
_vos_cDispEntitySize = .2 * _vos_cDispEntitySize
   endif
   else
   if VOS_cDispScope == VOS_eDispFar && _pic_maxrom > 03Fh
_vos_cDispEntitySize = _vos_cDispEntitySize + .1
   endif
   if VOS_cDispScope == VOS_eDispFar && _pic_maxrom > 07Fh
_vos_cDispEntitySize = _vos_cDispEntitySize + .1    
   endif
    
   endif

vos_CallInit	
    vos_mLoadPCLA
    movfw vos_currClass
    addlw   _vos_cDispEntitySize
    skpnc
    incf    PCLATH,F
    movwf   PCL

vos_CallHandler    
    vos_mLoadPCLA
    movfw   vos_currClass
    movwf   PCL

;------------------------------------------------------------------------------
; System initialization routines
;------------------------------------------------------------------------------
vos_InitSFR
    clrf        STATUS
    clrf        INTCON
    _pic_bankinit
    _pic_banksel PIR1
    clrf        PIR1
    ifdef       PIR2
    clrf        PIR2
    endif
    ifdef       RCON
    clrf        RCON
    bsf         RCON, IPEN
    endif
                                errorlevel -302
    _pic_banksel PIE1
    clrf        PIE1
    ifdef       PIE2
    _pic_banksel PIE2
    clrf        PIE2
    endif
                                errorlevel +302
    clrf        VOS_FLAGS
    bsf         VOS_bQueueEmpty
    return

;------------------------------------------------------------------------------
vos_InitWDT
 errorlevel -302
 ifdef WDTCON
    _sfr_banksel WDTCON 
    clrf WDTCON         ; Clear prescaler, WDT timeout 1 ms
  ifdef SWDTEN
    bsf WDTCON,SWDTEN
  endif
 endif
 ; if no prescaler available, WDT timeout 18 ms
 ifdef OPTION_REG
    _sfr_banksel OPTION_REG
    movlw   0F0h
    andwf   OPTION_REG, F   ; clear postscaler
    return
  endif
 errorlevel +302

;------------------------------------------------------------------------------
vos_InitQueue
    banksel vos_putptr
    movlw   low(vos_messageQueue)
    movwf   vos_putptr
    movwf   vos_msgptr
    return
;------------------------------------------------------------------------------
VOS_ObjectSectionEnd code
vos_objectSectionEnd
;------------------------------------------------------------------------------
; VOS object initialization
;------------------------------------------------------------------------------
VOS_BootHndlStrt code
vos_InitSystem
VOS_BootHndlDone code
vos_InitObjects
 if VOS_cDEBUG
    movlw   scnstart_high VOS_ObjectSection
    andlw   20h
   if VOS_cPersistentLocation  == VOS_ePLRom
    skpz
   else
    skpnz
   endif
vos_trapBadRegistry
    bra     $    
 endif
    banksel vos_currObj
    loadf   vos_currObj, scnstart_low VOS_ObjectSection

    ifdef vos_idleObj
    movff    vos_currObj, vos_idleObj
    endif
io1 rcall   vos_GetNextObj
    skpnc
    return
    skpz
    rcall   vos_CallInit
    pagesel vos_Operational
    banksel vos_currObj
    incf    vos_currObj, F
    incf    vos_currObj, F
    bra     io1

;------------------------------------------------------------------------------
;  System interrupt routine
;------------------------------------------------------------------------------
 if (_pic_fam == _pic16) || (_pic_fam == _pic12)
VOS_Vectors code
    org     0x04			; can not use goto at first place since selected page is not known
	movwf   TMP_WREG        ; 3: save W
    org     0x05
	swapf   STATUS, W       ; 4: load STATUS
	movwf   TMP_STATUS      ; 5: save STATUS
    movff   FSR, TMP_FSR    ; 6,7
    if _pic_maxrom >= 0800h ; 
    movff   PCLATH, TMP_PCLATH
    clrf    PCLATH          ; ISR are forced to page0 by the linker scripts (VOS_Interrupts section)
    endif
    goto    vos_EnterInterrupt

VOS_Interrupts code
vos_EnterInterrupt

VOS_InterruptDone code
    global  vos_LeaveInterrupt
vos_LeaveInterrupt
    movff   TMP_FSR, FSR
	swapf   TMP_STATUS, W   ; 
	movwf   STATUS          ; restore STATUS
	swapf   TMP_WREG, F     ; 
	swapf   TMP_WREG, W     ; restore W
vos_HandledInterrupt
    retfie
 endif

 if _pic_fam == _pic18
VOS_Vectors code
    org     0x08
    movff   FSR0L, TMP_FSR0L   ; High priority interrupt can be nested in low-priority
    movff   FSR0H, TMP_FSR0H   ; Therefore it uses its own registers for saving FSR, BSR
    movff   BSR, TMP_BSR0      ; BSR can be overwritten in PostMessage
    bra     VOS_HighLvl

    org     0x18
	movwf   TMP_WREG        ; 3: save W
	swapf   STATUS, W       ; 4: load STATUS
	movwf   TMP_STATUS      ; 5: save STATUS
    movff   FSR0L, TMP_FSR  ; 6,7
    movff   FSR0H, TMP_FSRH ; 8,9
    movff   BSR, TMP_BSR    ; 10,11: BSR can be overwritten in PostMessage
    bra     VOS_LowLvl

VOS_HighLevel code
VOS_HighLvl
VOS_HighLevelDone code
    movff   TMP_BSR0, BSR
    movff   TMP_FSR0H, FSR0H
    movff   TMP_FSR0L, FSR0L
    retfie  FAST
    code

VOS_LowLevel code
VOS_LowLvl
VOS_LowLevelDone code
    global  vos_LeaveInterrupt
vos_LeaveInterrupt
    movff   TMP_BSR,  BSR
    movff   TMP_FSR,  FSR0L
    movff   TMP_FSRH, FSR0H
	swapf   TMP_STATUS, W   ; 
	movwf   STATUS          ; restore STATUS
	swapf   TMP_WREG, F     ; 
	swapf   TMP_WREG, W     ; restore W
vos_HandledInterrupt
    retfie
 endif
 END
